#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#

import os, fcntl
from snake.xmlhelper import indent, ET

class LabIndex(object):
    '''
    LabIndex provides simple add / delete / getindex operations 
    on the top of lab's index.xml'''
    def __init__(self, dbroot):
        self.index = "%s/index.xml" % dbroot
        self.root = ET.Element('lab')
        self.root.attrib['xmlns:xi']='http://www.w3.org/2001/XInclude'

        if os.path.exists(self.index) and os.stat(self.index)[6]:
            try:
                r = ET.parse(self.index).getroot()
            except:
                pass
            else:
                for child in r.getchildren():
                    subelem = ET.SubElement(self.root, 'xi:include')
                    subelem.attrib['href'] = child.attrib['href']

    def _write_index(self):
        file = open(self.index, 'w')
        # set an advisory file lock
        fcntl.flock(file.fileno(), fcntl.LOCK_EX)
        file.write(ET.tostring(indent(self.root)))
        # release the file lock
        fcntl.flock(file.fileno(), fcntl.LOCK_UN)
        file.close()

    def init_empty_index(self):
        del self.root
        self.root = ET.Element('lab')
        self.root.attrib['xmlns:xi']='http://www.w3.org/2001/XInclude'
        self._write_index()
        return

    # add another <xi:include href=entry /> to the lab index
    def add(self, entry):
        '''
        Usage: LabIndex.add(entry)
        Description: add new <xi:include href=entry /> line
                     to the lab's index.xml'''
        subelem = ET.SubElement(self.root, 'xi:include')
        subelem.attrib['href'] = entry.replace("//", "/")
        self._write_index()

    # delete all <xi:include href=entry /> from the lab index
    def delete(self, entry):
        '''
        Usage: LabIndex.delete(entry)
        Description: remove all matching <xi:include href=entry> lines
                     from the lab's index.xml'''
        for child in self.root.getchildren():
            if child.attrib['href'] == entry:
                self.root.remove(child)
        self._write_index()

    # return the list of @href for every <xi:include href="..." /> line
    def getindex(self):
        '''
        Usage: LabIndex.getindex()
        Description: get list of URLs of all files included from lab's
                     index.xml'''
        ret = []
        for child in self.root.getchildren():
            if child.attrib.has_key('href'):
                ret.append(child.attrib['href'])
        return ret
